// PongSimu.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "PongSimu.h"
#include "avr-stuff.h"

// display orientation landscape if defined, else portrait
//#define LANDSCAPE

#ifdef LANDSCAPE
// width of display (LED count)
#define WIDTH 12
// height of display (LED count)
#define HEIGHT 10
#else
// width of display (LED count)
#define WIDTH 10
// height of display (LED count)
#define HEIGHT 12
#endif

struct Bitmaps
{
	HBITMAP frame;
	HBITMAP LED_on;
	HBITMAP LED_halve;
	HBITMAP LED_off;
};
// bitmap for LED visualisation
const int TileSize= 8;
const int InterSpacing= 6;

struct PaintParams
{
	HDC dc;
	int x0;
	int y0;
	CRITICAL_SECTION csForVars;
	bool endFlag;
	HANDLE hDoPaint;
	HANDLE hThread;
};

// Global Variables:
HINSTANCE g_hInst;								// current instance
HWND g_hWnd;                      // main window
HWND g_hWndSlider[2];             // slider windows
WNDPROC g_procSlider[2];          // slider windows old wndproc
HWND g_hWndHidden;             // hidden wnd so focus / accelerator stuff works...
LPCSTR szTitle= "PongSimu";				// The title bar text
LPCSTR szWindowClass= "PONGSIMU";	// the main window class name
Bitmaps s_bitmaps;
PaintParams s_paintparams;

// global Pixel buffer (always anode, cathode order as this is optimal for the multiplexer)
static byte g_pixbuf[10][12];
// inital ADC values --> knobs position is in range 0..1023
// left poti if turned max right, right poti if turned max left would give 1023 (at Vcc)
static uint g_adcval[2]= { 1023-500, 1023-500 }; 
static CRITICAL_SECTION g_csForADC;
volatile byte g_avrEnd;
static HANDLE g_hThread;

// v is the brightness code, use only 0, 1, and 2, while 2 be the brightest
void avrSetPixel(byte x, byte y, byte v)
{
	if (x >= WIDTH) DebugBreak();
	if (y >= HEIGHT) DebugBreak();
#ifdef LANDSCAPE
	g_pixbuf[y][x]= v;
#else
	g_pixbuf[x][y]= v;
#endif
}
byte avrGetPixel(byte x, byte y)
{
	if (x >= WIDTH) DebugBreak();
	if (y >= HEIGHT) DebugBreak();
#ifdef LANDSCAPE
	return g_pixbuf[y][x];
#else
	return g_pixbuf[x][y];
#endif
}
void avrTriggerRepaint()
{
	SetEvent(s_paintparams.hDoPaint);
}
// should we end the RunAVR loop (app exit on windose)
byte avrEndRun()
{
	return g_avrEnd;
}

// get the simulated ADC values
uint avrGetLeftADC()
{
	EnterCriticalSection(&g_csForADC);
	uint v= g_adcval[0];
	LeaveCriticalSection(&g_csForADC);
	return v;
}
uint avrGetRightADC()
{
	EnterCriticalSection(&g_csForADC);
	uint v= g_adcval[1];
	LeaveCriticalSection(&g_csForADC);
	return v;
}

// the use of the QueryPerformanceCounter Function is borrowed somehow from Microsoft, so it must be correct...
// see http://msdn.microsoft.com/en-us/library/ms644904(VS.85).aspx
extern double m_frequency;

class HRTimer {
public:
	static double GetFrequency() {
		LARGE_INTEGER proc_freq;
		if (!::QueryPerformanceFrequency(&proc_freq)) DebugBreak();
		return 1.0 / proc_freq.QuadPart;
	}
	HRTimer() {
		DWORD_PTR oldmask = ::SetThreadAffinityMask(::GetCurrentThread(), 0);
		::QueryPerformanceCounter(&start);
	}
	double GetTimeSince() {
	  LARGE_INTEGER curr;
		::QueryPerformanceCounter(&curr);
		return (curr.QuadPart - start.QuadPart) * m_frequency;
	}
	~HRTimer() {
		::SetThreadAffinityMask(::GetCurrentThread(), m_oldmask);
	}
private:
  LARGE_INTEGER start;
	DWORD_PTR m_oldmask;
};

double m_frequency= HRTimer::GetFrequency();

// simulation of some AVR lib funcs
void _delay_us(double __us)
{
	if (__us <= 0) DebugBreak();
	if (__us < 1000) {
		__us /= (1000*1000);
		volatile unsigned __int64 i= 0;
		HRTimer hrt;
		while (hrt.GetTimeSince() < __us)
			++i;
	}
	else
		Sleep(static_cast<DWORD>(__us / 1000));
}

void _delay_ms(double __ms)
{
	if (__ms <= 0) DebugBreak();
	Sleep(static_cast<DWORD>(__ms));
}

uint ticks(void)
{
	return (uint)GetTickCount();
}


// thread function to start the avr Thread
static unsigned int _stdcall ThreadFunc(void* args)
{
	//Sleep(1000); // 1 sec until simulated avr starts so user can change initial knobs position
	RunAVR();
	return 0;
}


// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK	AboutDlgProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK	KnobHelpDlgProc(HWND, UINT, WPARAM, LPARAM);


int APIENTRY WinMain(HINSTANCE hInstance,
										 HINSTANCE hPrevInstance,
										 LPTSTR    lpCmdLine,
										 int       nCmdShow)
{
	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	{
	INITCOMMONCONTROLSEX icc= { sizeof(icc), ICC_BAR_CLASSES };
	InitCommonControlsEx(&icc); // loads common control's DLL 
	}

	// Initialize global strings
	MyRegisterClass(hInstance);

	hAccelTable = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDC_PONGSIMU));

	s_bitmaps.frame= (HBITMAP)LoadImage(hInstance, MAKEINTRESOURCE(IDB_FRAME),IMAGE_BITMAP, 0, 0, LR_DEFAULTSIZE|LR_CREATEDIBSECTION);  
	s_bitmaps.LED_off=   (HBITMAP)LoadImage(hInstance, MAKEINTRESOURCE(IDB_LEDOFF),IMAGE_BITMAP, 0, 0, LR_DEFAULTSIZE|LR_CREATEDIBSECTION);  
	s_bitmaps.LED_halve= (HBITMAP)LoadImage(hInstance, MAKEINTRESOURCE(IDB_LEDHALVE),IMAGE_BITMAP, 0, 0, LR_DEFAULTSIZE|LR_CREATEDIBSECTION);  
	s_bitmaps.LED_on=    (HBITMAP)LoadImage(hInstance, MAKEINTRESOURCE(IDB_LEDON),IMAGE_BITMAP, 0, 0, LR_DEFAULTSIZE|LR_CREATEDIBSECTION);  

  // AVR thread
	InitializeCriticalSection(&g_csForADC);
	g_hThread= (HANDLE)_beginthreadex(NULL, 0, ThreadFunc, NULL, CREATE_SUSPENDED, NULL);
	SetThreadPriority(g_hThread, THREAD_PRIORITY_BELOW_NORMAL);
	ResumeThread(g_hThread);

	// paint thread prepare
	s_paintparams.hDoPaint= CreateEvent(NULL, false, false, NULL);
	InitializeCriticalSection(&s_paintparams.csForVars);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow))
		return FALSE;

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		if (!TranslateAccelerator(g_hWnd, hAccelTable, &msg))	{
			if (!IsDialogMessage(g_hWnd, &msg)) {
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
	}

	g_avrEnd= true;
	WaitForSingleObject(g_hThread, INFINITE);
	CloseHandle(g_hThread);
	return (int) msg.wParam;
}



//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function registers the class, together with their icons
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_PONGSIMU));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOWTEXT+1);
	wcex.lpszMenuName	= MAKEINTRESOURCE(IDC_PONGSIMU);
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= wcex.hIcon;

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HINSTANCE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	g_hInst = hInstance; // Store instance handle in our global variable

	RECT r= { 0, 0, 300, 300 };
	AdjustWindowRect(&r, WS_OVERLAPPEDWINDOW, true);

	g_hWnd= CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
												 //CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);
												 400, 300, r.right-r.left, r.bottom-r.top, NULL, NULL, hInstance, NULL);

	if (!g_hWnd)
		return FALSE;

	ShowWindow(g_hWnd, nCmdShow);
	//UpdateWindow(hWnd);
	return TRUE;
}

static unsigned int _stdcall PaintThreadFunc(void* args)
{
	HDC hdc= s_paintparams.dc;
	HDC hdcMem= CreateCompatibleDC(hdc);

	for (;;) {
		WaitForSingleObject(s_paintparams.hDoPaint, INFINITE);
		if (s_paintparams.endFlag) break;
		EnterCriticalSection(&s_paintparams.csForVars);
		int x0= s_paintparams.x0;
		int y0= s_paintparams.y0;
		LeaveCriticalSection(&s_paintparams.csForVars);

		for (int y= 0; y < HEIGHT; ++y) {
			for (int x= 0; x < WIDTH; ++x) {
				byte b= avrGetPixel(x, HEIGHT-1-y);
				if (b==0)
					SelectObject(hdcMem, s_bitmaps.LED_off);
				else if (b==1)
					SelectObject(hdcMem, s_bitmaps.LED_halve);
				else
					SelectObject(hdcMem, s_bitmaps.LED_on);
				BitBlt(hdc, x0 + (TileSize+InterSpacing) * x, y0 + (TileSize+InterSpacing) * y,
							 TileSize, TileSize, hdcMem, 0, 0, SRCCOPY);
			}
		}
	}
	DeleteDC(hdcMem);
	return 0;
}

static void MoveSlider(HWND wnd, HWND slider, bool up)
{
	SendMessage(slider, TBM_SETPOS, true, SendMessage(slider, TBM_GETPOS, 0, 0) + (up ? - 32 : 32));
	SendMessage(wnd, WM_VSCROLL, 0, (LPARAM) slider);
}

static void SetDlgItemFocus(HWND wnd, HWND hFocus)
{
	SetFocus(hFocus);
	PostMessage(wnd, WM_NEXTDLGCTL, (WPARAM)hFocus, 1);
}

static void HandleFocus(HWND hWnd, bool get)
{
	static int focus= 0;
	if (!get) {
		HWND hFocus= GetFocus();
		if (hFocus==g_hWndSlider[0]) focus= 0;
		else if (hFocus==g_hWndSlider[1]) focus= 1;
		else focus= 2;
	}
	else {
		if (focus < 2)
			SetDlgItemFocus(hWnd, g_hWndSlider[focus]);
		else
			SetDlgItemFocus(hWnd, g_hWndHidden);
	}
}

// subclass the slider, only to avoid itself drawing its background with default color
// we want the window background here
LRESULT CALLBACK NewSliderWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (uMsg == WM_ERASEBKGND)
		return 1;

	if (hwnd == g_hWndSlider[0])
		return CallWindowProc(g_procSlider[0], hwnd, uMsg, wParam, lParam);
	return CallWindowProc(g_procSlider[1], hwnd, uMsg, wParam, lParam);
}

//
//  FUNCTION: WndProc(HWND, UINT, WPARAM, LPARAM)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	const int FrameSize= 16;
	const int FrameSpace= 10;
	const int FrameSizeSpace= FrameSize + FrameSpace;
	const int XPong= (TileSize+InterSpacing) * WIDTH - InterSpacing;
	const int YPong= (TileSize+InterSpacing) * HEIGHT - InterSpacing;
	const int XFrame= XPong + 2*FrameSizeSpace;
	const int YFrame= YPong + 2*FrameSizeSpace;
	const int SliderWidht= 20;
	const int SliderSpace= 14;
	const int SliderWidthSpace= SliderWidht + SliderSpace;
	const int SliderToClient= 40;

	switch (message)
	{
	case WM_COMMAND: {
		int wmId    = LOWORD(wParam);
		int wmEvent = HIWORD(wParam);
		// Parse the menu selections:
		switch (wmId)
		{
		case IDM_KNOBS: {
			static int focus= 0;
			if (IsWindowVisible(g_hWndSlider[0])) {
				HandleFocus(hWnd, false);
				SetFocus(g_hWndHidden);
				ShowWindow(g_hWndSlider[0], SW_HIDE);
				ShowWindow(g_hWndSlider[1], SW_HIDE);
			}
			else {
				ShowWindow(g_hWndSlider[0], SW_SHOW);
				ShowWindow(g_hWndSlider[1], SW_SHOW);
				HandleFocus(hWnd, true);
			}
			break; }
		case IDM_L_UP:
			MoveSlider(hWnd, g_hWndSlider[0], true);
			break;
		case IDM_L_DN:
			MoveSlider(hWnd, g_hWndSlider[0], false);
			break;
		case IDM_R_UP:
			MoveSlider(hWnd, g_hWndSlider[1], true);
			break;
		case IDM_R_DN:
			MoveSlider(hWnd, g_hWndSlider[1], false);
			break;
		case IDM_EXIT:
			SendMessage(hWnd, WM_CLOSE, 0, 0);
			break;
		case IDM_ABOUT: {
			HWND hFocus= GetFocus();
			DialogBox(g_hInst, MAKEINTRESOURCE(IDD_ABOUTBOX), hWnd, AboutDlgProc);
			SetDlgItemFocus(hWnd, hFocus);
			break; }
		case IDM_ONKNOBS: {
			HWND hFocus= GetFocus();
			DialogBox(g_hInst, MAKEINTRESOURCE(IDD_KNOBHELP), hWnd, KnobHelpDlgProc);
			SetDlgItemFocus(hWnd, hFocus);
			break; }
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break; }

	case WM_CTLCOLORSTATIC: {
		HDC hdc= (HDC)wParam;
		return (LRESULT) GetStockObject(BLACK_BRUSH); }

	case WM_NOTIFY: {
		NMCUSTOMDRAW* notify = (NMCUSTOMDRAW*) lParam;
		if (notify->hdr.hwndFrom == g_hWndSlider[0] || notify->hdr.hwndFrom == g_hWndSlider[1]) {
			if (notify->dwDrawStage == CDDS_PREPAINT) {
				if (notify->uItemState & CDIS_FOCUS) {
					notify->uItemState &= ~CDIS_FOCUS;
					RECT r;
					GetClientRect(notify->hdr.hwndFrom, &r);
#if 0  // correct focus draw, but with longer dashes looks better...
					const LOGBRUSH br= { BS_SOLID, RGB(0,170,0) };
					const DWORD style[2]={0,0};
					HPEN pen= ExtCreatePen(PS_COSMETIC | PS_ALTERNATE, 1, &br, 0, NULL);
#else
					HPEN pen= CreatePen(PS_DOT, 1, RGB(0,170,0));
#endif
					HGDIOBJ oldpen= SelectObject(notify->hdc, pen);
					SetBkColor(notify->hdc, RGB(0,0,0));
					HBRUSH oldbr= (HBRUSH)SelectObject(notify->hdc, GetStockObject(NULL_BRUSH));
					Rectangle(notify->hdc, r.left, r.top, r.right, r.bottom);
					SelectObject(notify->hdc, oldpen);
					SelectObject(notify->hdc, oldbr);
					DeleteObject(pen);
				}
				return CDRF_NOTIFYITEMDRAW;
			}
			if (notify->dwDrawStage == CDDS_ITEMPREPAINT && notify->dwItemSpec == TBCD_TICS) {
				HPEN pen= CreatePen(PS_SOLID, 0, RGB(0,170,0));
				RECT r;
				GetClientRect(notify->hdr.hwndFrom, &r);
				HGDIOBJ oldpen= SelectObject(notify->hdc, pen);
				const int YTop= 8;
				const int YBott= 11;
				const int TickStart= 16;
				const int TickWidth= 4;
				const int Mul= 1;
				const int NTicks= 8;
				const int RectHeight= r.bottom - r.top - YTop - YBott;
				for (int i= 0; i < NTicks+1; ++i) {
					int y= r.top + YTop + (i * RectHeight+NTicks-1)/NTicks;
					MoveToEx(notify->hdc, TickStart, y, NULL);
					LineTo(notify->hdc, TickStart + TickWidth, y);
				}
				SelectObject(notify->hdc, oldpen);
				DeleteObject(pen);
				return CDRF_SKIPDEFAULT;
			}
			return CDRF_DODEFAULT;
		}
		break; }
	case WM_PAINT: {
		PAINTSTRUCT ps;
		HDC hdc = BeginPaint(hWnd, &ps);

		// paint the yellow frame around
		HDC hdcMem = CreateCompatibleDC(hdc);

		SelectObject(hdcMem, s_bitmaps.frame);
		// top
		StretchBlt(hdc, s_paintparams.x0 - FrameSizeSpace, s_paintparams.y0 - FrameSizeSpace,
			 XPong + 2*FrameSizeSpace, FrameSize, hdcMem, 0, 0, TileSize, TileSize, SRCCOPY);

		// bottom
		StretchBlt(hdc, s_paintparams.x0 - FrameSizeSpace, s_paintparams.y0 + YPong + FrameSpace,
			 XPong + 2*FrameSizeSpace, FrameSize, hdcMem, 0, 0, TileSize, TileSize, SRCCOPY);

		// left
		StretchBlt(hdc, s_paintparams.x0 - FrameSizeSpace, s_paintparams.y0 - FrameSizeSpace,
			 FrameSize, YPong + 2*FrameSizeSpace, hdcMem, 0, 0, TileSize, TileSize, SRCCOPY);

		// right
		StretchBlt(hdc, s_paintparams.x0 + XPong + FrameSpace, s_paintparams.y0 - FrameSizeSpace,
			 FrameSize, YPong + 2*FrameSizeSpace, hdcMem, 0, 0, TileSize, TileSize, SRCCOPY);

		DeleteDC(hdcMem);
		EndPaint(hWnd, &ps);
		SetEvent(s_paintparams.hDoPaint);
		break; }

	case WM_VSCROLL: {
		HWND hScroll= (HWND)lParam;
		uint val= (uint)SendMessage(hScroll,TBM_GETPOS,0,0);
		uint idx= hScroll == g_hWndSlider[0] ? 0 : 1;
		if (g_adcval[idx] != val) { // read is safe here..
			EnterCriticalSection(&g_csForADC);
			g_adcval[idx]= val;
			LeaveCriticalSection(&g_csForADC);
		}
		break; }

	case WM_SIZE:
		if (wParam != SIZE_MINIMIZED) {
			RECT r;
			GetClientRect(hWnd, &r);
			if (r.right > 0 && r.bottom > 0) {

				// pong field left-top
				int x0= (r.right-r.left-XPong)/2;
				int y0= (r.bottom-r.top-YPong)/2;

				// adjust pos of sliders
				int xs= x0 - FrameSizeSpace - SliderWidthSpace;
				int ys= y0 - FrameSizeSpace;
				if (xs > SliderToClient)
					xs= SliderToClient;
				SetWindowPos(g_hWndSlider[0], NULL, xs, ys, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
				xs= x0 + XPong + FrameSizeSpace + SliderSpace;
				if (xs < r.right - SliderToClient - SliderWidht)
					xs= r.right - SliderToClient - SliderWidht;
				SetWindowPos(g_hWndSlider[1], NULL, xs, ys, 0, 0, SWP_NOZORDER | SWP_NOSIZE);

				// center the Pong into the client Rect
				EnterCriticalSection(&s_paintparams.csForVars);
				s_paintparams.x0= x0;
				s_paintparams.y0= y0;
				LeaveCriticalSection(&s_paintparams.csForVars);
			}
		}
		break;

	case WM_CREATE:
		// hidden button, to have something with focus (else IsDialogMessage does not translate menu items if not child has focus)
		g_hWndHidden= CreateWindowEx(0, "BUTTON", "x",
			  WS_CHILD | WS_TABSTOP,  // style 
				0,0,0,0,hWnd, (HMENU)99, g_hInst, NULL);
		// client controls (sliders)
		g_hWndSlider[0]= CreateWindowEx(0, TRACKBAR_CLASS, "Left",
			  WS_CHILD | WS_VISIBLE | WS_TABSTOP | TBS_VERT | TBS_AUTOTICKS,  // style 
				0,0,SliderWidht,YFrame,hWnd, (HMENU)100, g_hInst, NULL);
		g_hWndSlider[1]= CreateWindowEx(0, TRACKBAR_CLASS, "Right",
			  WS_CHILD | WS_VISIBLE | WS_TABSTOP | TBS_VERT,  // style 
				0,0,SliderWidht,YFrame,hWnd, (HMENU)101, g_hInst, NULL);
		for (int i= 0; i < 2; ++i) {
			// subclass them
			g_procSlider[i]= (WNDPROC)SetWindowLongPtr(g_hWndSlider[i], GWLP_WNDPROC, (LONG_PTR)NewSliderWndProc);

			SendMessage(g_hWndSlider[i], TBM_SETRANGE, true, MAKELPARAM(0,1023));  // min. & max. positions 
			SendMessage(g_hWndSlider[i], TBM_SETPAGESIZE, 0, 64);  // page size
			SendMessage(g_hWndSlider[i], TBM_SETTICFREQ, 128, 0);  // tick frequ
		}
		SendMessage(g_hWndSlider[0], TBM_SETPOS, true, g_adcval[0]);  // position
		SendMessage(g_hWndSlider[1], TBM_SETPOS, true, g_adcval[1]);  // position

		// intial selected slider
		SetFocus(g_hWndSlider[1]);

		// paint thread
		s_paintparams.dc= GetDC(hWnd);
		s_paintparams.hThread= (HANDLE)_beginthreadex(NULL, 0, PaintThreadFunc, NULL, 0, NULL);
		break;

	case WM_ACTIVATE:
		HandleFocus(hWnd, LOWORD(wParam) != WA_INACTIVE);
		break;

	case WM_DESTROY:
		// remove subclass
		for (int i= 0; i < 2; ++i)
			SetWindowLongPtr(g_hWndSlider[i], GWLP_WNDPROC, (LONG_PTR)g_procSlider[i]);

		s_paintparams.endFlag= true;
		SetEvent(s_paintparams.hDoPaint);
		WaitForSingleObject(s_paintparams.hThread, INFINITE);
		CloseHandle(s_paintparams.hThread);
		ReleaseDC(hWnd, s_paintparams.dc);
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// Message handler for about box.
INT_PTR CALLBACK AboutDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetDlgItemText(hDlg, IDC_TXT,
      "This application simulates the AVR hardware pong to some extent.\n"
		  "You can test code on Windows PC to later run on the AVR.\n"
			"Place your code in \"avr-stuff.c\" and enjoy ;-)"
			);
		return (INT_PTR)TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)	{
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		break;
	}
	return (INT_PTR)FALSE;
}

// Message handler for knob Help.
INT_PTR CALLBACK KnobHelpDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetDlgItemText(hDlg, IDC_KNOBTXT,
			"The Pong knobs are simulated by the two sliders left and right of the display.\n"
			"Clicking on menu bar item 'Knobs' shows or hides them.\n\n"
			"The focused knob can be moved with mouse, mouse wheel and the cursor keys.\n\n"
			"Also some keys move the knobs in step of 32, 'q' and 'a' move the left one,\n"
			"Numpad-[-] and Numpad-[+] move the right one.\n\nThese keys even work if the knobs are hidden.\n\n"
			"Functions avrGetLeftADC(), avrGetRightADC() return the position of the simulated knobs as value 0..1023."
			);
		return (INT_PTR)TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)	{
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		break;
	}
	return (INT_PTR)FALSE;
}
