/*
 * gameoflife.c
 *
 * Conways Game of life for Ping-Pong Board from Conrad
 *
 *
 * Ver.     Date         Author           Comments
 * -------  ----------   --------------   ------------------------------
 * 1.00-PC  06.03.2010   Hans Eckardt     adaption from the original sample code, see below
 * 1.00-PC  07.03.2010   Hans Eckardt     border can be set to >1, generation array recoded, faster code
 * 1.00-PC  16.03.2010   Hans Eckardt     small changes from real AVR program taken in the simulation
 *
 ***************************************************************************
 */

/****************************************************************************
** $Id: qt/life.cpp   3.3.2   edited May 27 2003 $
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/


/* ---------------------------------------------------------------------------- 
 * Includes																	 
 */

/*
#include <inttypes.h>		                   	
#include <avr/interrupt.h>                     	
#include <avr/io.h>
#include <util/delay.h>		                   	
#include <avr/sleep.h>
#include <avr/power.h>
#include <avr/pgmspace.h>                      	
*/

#include "avr-stuff.h"                           	// simulated hardware lives here

#include "patterns.h"                           	// some Patterns for life, see file for copyright notice.

#ifdef _DEBUG // emulation debugging
#include <stdio.h>
extern "C" __declspec(dllimport) void __stdcall OutputDebugStringA(const char* lpOutputString);
#pragma comment(lib,"kernel32")
#endif

// the visible playfield size 
enum { MAXSIZEX = 12, MAXSIZEY = 10 };

// Conways field is infinite, but we use a fixed size for our simple algorithm, by adding an invisible border
// invisible border must be at least 1, >1 needed for some predefined patterns
// invisible=5 is greatest to fit in data space of ATMEGA8, this all could be optimized 
// (bits instead of bools, no separate display pixbuf)
enum { INIVISIBLE= 5, INIVISIBLE2= 2*INIVISIBLE };

// the cell array
bool	cells[MAXSIZEX + INIVISIBLE2][MAXSIZEY + INIVISIBLE2];

// clear cell array
void clear()
{
	memset(cells, 0, sizeof(cells)); // zero out
}

// set cell alive (only the visible cells can be set)
void setCell( uint i, uint j )
{
	if ( i >= MAXSIZEX || j >= MAXSIZEY )
		return;
	cells[i+INIVISIBLE][j+INIVISIBLE] = true;
}

// next gen of cells
void nextGeneration()
{
	// next generation array (static to save stack space)
	static bool	cells_new[MAXSIZEX + INIVISIBLE2][MAXSIZEY + INIVISIBLE2];
	memset(cells_new, 0, sizeof(cells_new)); // zero out the border

	for ( uint i = 1; i < MAXSIZEX+INIVISIBLE2-1; i++ ) {
		for ( uint j = 1; j < MAXSIZEY+INIVISIBLE2-1; j++ ) {
			uint t =  cells[i - 1][j - 1]
							+ cells[i - 1][j]
							+ cells[i - 1][j + 1]
							+ cells[i][j - 1]
							+ cells[i][j + 1]
							+ cells[i + 1][j - 1]
							+ cells[i + 1][j]
							+ cells[i + 1][j + 1];

			cells_new[i][j] = t == 3 || (t == 2 && cells[i][j]);
		}
	}
	memcpy(cells, cells_new, sizeof(cells));
}

// initial random gen of cells
void randomInit()
{
	uint PlayExt= MAXSIZEX * MAXSIZEY;
	for (uint i = 0; i < PlayExt/2; ++i) {
		uint pos= rand();
		setCell(pos % MAXSIZEX, pos / (RAND_MAX/MAXSIZEX)); 
	}
}

// sync the pixels with the life game board
void UpdatePixels()
{
	for ( uint i = 0; i < MAXSIZEX; i++ )
		for ( uint j = 0; j < MAXSIZEY; j++ )
			avrSetPixel(i,j,cells[i+INIVISIBLE][j+INIVISIBLE] ? 2 : 0);
	avrTriggerRepaint();
}

// get a pattern to the life game
void getPattern( int pat )
{
	int i = pat % NPATS;
	int col;
	int * patptr = &patterns[i][0];
	while ( (col = *patptr++) != 127 ) {
		int row = *patptr++;
		col += MAXSIZEX / 2;
		row += MAXSIZEY / 2;
		setCell( col, row );
  }
}

// main loop for life game
void RunAVR_Life(void)
{
	uint LPotiP = avrGetLeftADC();
	uint RPotiP = avrGetRightADC()+11; // begin with a random pattern selection

	// wait for start of game
	_delay_ms(500);

	// Main Loop
	while (!avrEndRun()) {						// Typical loop for Controler

		UpdatePixels();
		uint RPoti = avrGetRightADC();
		uint LPoti = avrGetLeftADC();

		if (abs(RPotiP - RPoti) > 10) {
			RPotiP = RPoti;
			srand(RPoti+2); // avoid same pattern

			clear(); // clear the life board
			randomInit(); // init with random values
			UpdatePixels();
			_delay_ms(100);
			continue;
		}
		else if (abs(LPotiP - LPoti) > 10) {
			// todo: known pattern
			LPotiP = LPoti;
			clear(); // clear the life board
			getPattern( (1023-LPoti) / 80 ); // get a known pattern (11 patterns, so 1023/11=93 divide by 80)
			UpdatePixels();
			_delay_ms(800);
			continue;
		}
		else
			nextGeneration();
		_delay_ms(250);
	}
}
